==> gt4f90io-g95_20060627+20060717.tar.gz: gt4f90io-g95-20060627+20060717/src/gtvaropenbydimord.f90 <==
!== Open GT_VARIABLE of dimension by dimord
!
! Authors::   Yasuhiro MORIKAWA, Eizi TOYODA
! Version::   $Id: gtvaropenbydimord.f90,v 1.6 2006/06/16 04:55:09 morikawa Exp $
! Tag Name::  $Name: gt4f90io-20060627 $
! Copyright:: Copyright (C) GFD Dennou Club, 2000-2005. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!
! ʲΥ֥롼, ؿ gtdata_generic 󶡤ޤ
!

subroutine GTVarOpenByDimOrd(var, source_var, dimord, count_compact, err)
  !
  !== gtool4 ǡΥץ
  !
  ! ˳줿ѿ *source_var*  *dimord* ܤμˤѿ
  !  var ˳Ǽޤ *dimord* ϸߤϰϤ
  ! ˤʤäƤ (ѥȲƤ) Ф
  ! Ǥ*count_compact*  <tt>.true.</tt>ꤹ
  ! ٤ƤμΤʤνˤʤޤ
  !
  ! Open 줿ѿɬ Close ʤФʤޤ
  !
  ! *dimord* == 0 ξѿΤٳޤϻȥ󥿤
  ! äʤǤ
  !
  ! *Open*  2 ĤΥ֥롼̾Ǥꡢ
  ! ѿ URL ľܻꤹ뤳ȤǳȤǽǤ
  ! Υ֥롼򻲾Ȥ
  !
  !=== ­
  !
  ! ѿ URL ˥ե̾ꤷʤ硢gtool.nc ǤȤߤʤޤ
  !
  ! ѿ URL ˥ե̾ꤷ硢ѿϰʲε§
  ! 򤵤ޤ
  !
  ! * ѿ򤵤ʤ
  ! * ʤ٤줿ѿ򤵤
  !
  use gtdata_types, only: GT_VARIABLE
  use gt_map, only: var_class, vtb_class_netcdf, vtb_class_memory, &
    & map_dup, map_create, map_lookup, GT_DIMMAP, &
    & map_set, dimord_skip_compact
  use an_generic, only: Open, AN_VARIABLE, inquire
  use gtdata_generic, only: gt_open => open
  use dc_present, only: present_and_true
  use dc_trace, only: beginsub, endsub, DbgMessage
  use dc_string, only: var_str, CPrintf
  use dc_error, only: StoreError, GT_ENOMOREDIMS, GT_ENOMOREDIMS, &
    & DC_NOERR, GT_EFAKE
  use dc_types, only: STRING
  implicit none
  type(GT_VARIABLE), intent(out):: var
  type(GT_VARIABLE), intent(in):: source_var
  integer, intent(in):: dimord
  logical, intent(in), optional:: count_compact
  logical, intent(out), optional:: err
  integer:: sclass, scid, ld, sndims, stat, udimord, idimord, cause_i
  type(AN_VARIABLE):: an
  type(GT_DIMMAP), allocatable:: map_src(:)
  type(GT_DIMMAP):: map_result(1)
  logical:: cnt_compact
  character(STRING) :: endsub_msg
  character(len = *),      parameter:: subname = "GTVarOpen-By-Dimord"
  character(len = *),      parameter:: version = &
    & '$Name: gt4f90io-20060627 $' // &
    & '$Id: gtvaropenbydimord.f90,v 1.6 2006/06/16 04:55:09 morikawa Exp $'
continue
  call beginsub(subname, 'var.mapid=%d dimord=%d ', &
    & i=(/source_var%mapid, dimord/), version=version)
  stat = DC_NOERR
  cause_i = dimord
  endsub_msg = ''

  ! ѿ켫Τ򳫤ľ
  if (dimord == 0) then
    call map_dup(var, source_var)
    if (present(err)) err = .false.
    endsub_msg = 'dup'
    goto 999
  endif

  ! ɽdimord  (count_compact ) μѿ
  ! ֹĴ٤롣
  call map_lookup(source_var, ndims=sndims)
  if (sndims <= 0 .or. dimord > sndims) then
    stat = GT_ENOMOREDIMS
    goto 999
  endif
  allocate(map_src(sndims))
  call map_lookup(source_var, map=map_src)
  cnt_compact = .false.
  if (present_and_true(count_compact)) then
    cnt_compact = .true.
  else
    cnt_compact = .false.
  end if
  call DbgMessage('count_compact=%y', l=(/cnt_compact/))

  if (cnt_compact) then
    udimord = dimord
  else
    udimord = dimord_skip_compact(dimord, map=map_src)
  endif
  if (udimord <= 0 .or. udimord > size(map_src)) then
    stat = GT_ENOMOREDIMS
    goto 999
  endif

  idimord = map_src(udimord)%dimno
  if (idimord < 1) then
    call gt_open(var, map_src(udimord)%url, err=err)
    ! storeerror ϤʤƤ褤
    deallocate(map_src)
    goto 999
  endif

  ! ּ̤˹碌ּѿץ׽
  call var_class(source_var, sclass, scid)
  if (sclass == vtb_class_netcdf) then
    call Open(an, AN_VARIABLE(scid), idimord, err)
    call inquire(an, dimlen=ld)
    call map_create(var, vtb_class_netcdf, an%id, 1, (/ld/), stat)
    if (stat /= DC_NOERR) then
      cause_i = 1
      goto 999
    end if
    call map_lookup(var, map=map_result)
    map_result(1)%offset = map_src(udimord)%offset
    map_result(1)%step = map_src(udimord)%step
    map_result(1)%allcount = map_src(udimord)%allcount
    map_result(1)%start = map_src(udimord)%start
    map_result(1)%count = map_src(udimord)%count
    map_result(1)%stride = map_src(udimord)%stride
    call map_set(var, map=map_result, stat=stat)
  else if (sclass == vtb_class_memory) then
    var = source_var
    stat = DC_NOERR
  else
    stat = GT_EFAKE
  endif

  deallocate(map_src)
  endsub_msg = CPrintf('result_var=%d', i=(/var%mapid/))
999 continue
  call StoreError(stat, subname, cause_i=cause_i)
  call endsub(subname, '%c', c1=trim(endsub_msg))
end subroutine GTVarOpenByDimOrd

==> gt4f90io-g95_20060719-1+20060721.tar.gz: gt4f90io-g95-20060719-1+20060721/src/gtvaropenbydimord.f90 <==
!== Open GT_VARIABLE of dimension by dimord
!
! Authors::   Yasuhiro MORIKAWA, Eizi TOYODA
! Version::   $Id: gtvaropenbydimord.f90,v 1.6 2006/06/16 04:55:09 morikawa Exp $
! Tag Name::  $Name: gt4f90io-20060719-1 $
! Copyright:: Copyright (C) GFD Dennou Club, 2000-2005. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!
! ʲΥ֥롼, ؿ gtdata_generic 󶡤ޤ
!

subroutine GTVarOpenByDimOrd(var, source_var, dimord, count_compact, err)
  !
  !== gtool4 ǡΥץ
  !
  ! ˳줿ѿ *source_var*  *dimord* ܤμˤѿ
  !  var ˳Ǽޤ *dimord* ϸߤϰϤ
  ! ˤʤäƤ (ѥȲƤ) Ф
  ! Ǥ*count_compact*  <tt>.true.</tt>ꤹ
  ! ٤ƤμΤʤνˤʤޤ
  !
  ! Open 줿ѿɬ Close ʤФʤޤ
  !
  ! *dimord* == 0 ξѿΤٳޤϻȥ󥿤
  ! äʤǤ
  !
  ! *Open*  2 ĤΥ֥롼̾Ǥꡢ
  ! ѿ URL ľܻꤹ뤳ȤǳȤǽǤ
  ! Υ֥롼򻲾Ȥ
  !
  !=== ­
  !
  ! ѿ URL ˥ե̾ꤷʤ硢gtool.nc ǤȤߤʤޤ
  !
  ! ѿ URL ˥ե̾ꤷ硢ѿϰʲε§
  ! 򤵤ޤ
  !
  ! * ѿ򤵤ʤ
  ! * ʤ٤줿ѿ򤵤
  !
  use gtdata_types, only: GT_VARIABLE
  use gt_map, only: var_class, vtb_class_netcdf, vtb_class_memory, &
    & map_dup, map_create, map_lookup, GT_DIMMAP, &
    & map_set, dimord_skip_compact
  use an_generic, only: Open, AN_VARIABLE, inquire
  use gtdata_generic, only: gt_open => open
  use dc_present, only: present_and_true
  use dc_trace, only: beginsub, endsub, DbgMessage
  use dc_string, only: var_str, CPrintf
  use dc_error, only: StoreError, GT_ENOMOREDIMS, GT_ENOMOREDIMS, &
    & DC_NOERR, GT_EFAKE
  use dc_types, only: STRING
  implicit none
  type(GT_VARIABLE), intent(out):: var
  type(GT_VARIABLE), intent(in):: source_var
  integer, intent(in):: dimord
  logical, intent(in), optional:: count_compact
  logical, intent(out), optional:: err
  integer:: sclass, scid, ld, sndims, stat, udimord, idimord, cause_i
  type(AN_VARIABLE):: an
  type(GT_DIMMAP), allocatable:: map_src(:)
  type(GT_DIMMAP):: map_result(1)
  logical:: cnt_compact
  character(STRING) :: endsub_msg
  character(len = *),      parameter:: subname = "GTVarOpen-By-Dimord"
  character(len = *),      parameter:: version = &
    & '$Name: gt4f90io-20060719-1 $' // &
    & '$Id: gtvaropenbydimord.f90,v 1.6 2006/06/16 04:55:09 morikawa Exp $'
continue
  call beginsub(subname, 'var.mapid=%d dimord=%d ', &
    & i=(/source_var%mapid, dimord/), version=version)
  stat = DC_NOERR
  cause_i = dimord
  endsub_msg = ''

  ! ѿ켫Τ򳫤ľ
  if (dimord == 0) then
    call map_dup(var, source_var)
    if (present(err)) err = .false.
    endsub_msg = 'dup'
    goto 999
  endif

  ! ɽdimord  (count_compact ) μѿ
  ! ֹĴ٤롣
  call map_lookup(source_var, ndims=sndims)
  if (sndims <= 0 .or. dimord > sndims) then
    stat = GT_ENOMOREDIMS
    goto 999
  endif
  allocate(map_src(sndims))
  call map_lookup(source_var, map=map_src)
  cnt_compact = .false.
  if (present_and_true(count_compact)) then
    cnt_compact = .true.
  else
    cnt_compact = .false.
  end if
  call DbgMessage('count_compact=%y', l=(/cnt_compact/))

  if (cnt_compact) then
    udimord = dimord
  else
    udimord = dimord_skip_compact(dimord, map=map_src)
  endif
  if (udimord <= 0 .or. udimord > size(map_src)) then
    stat = GT_ENOMOREDIMS
    goto 999
  endif

  idimord = map_src(udimord)%dimno
  if (idimord < 1) then
    call gt_open(var, map_src(udimord)%url, err=err)
    ! storeerror ϤʤƤ褤
    deallocate(map_src)
    goto 999
  endif

  ! ּ̤˹碌ּѿץ׽
  call var_class(source_var, sclass, scid)
  if (sclass == vtb_class_netcdf) then
    call Open(an, AN_VARIABLE(scid), idimord, err)
    call inquire(an, dimlen=ld)
    call map_create(var, vtb_class_netcdf, an%id, 1, (/ld/), stat)
    if (stat /= DC_NOERR) then
      cause_i = 1
      goto 999
    end if
    call map_lookup(var, map=map_result)
    map_result(1)%offset = map_src(udimord)%offset
    map_result(1)%step = map_src(udimord)%step
    map_result(1)%allcount = map_src(udimord)%allcount
    map_result(1)%start = map_src(udimord)%start
    map_result(1)%count = map_src(udimord)%count
    map_result(1)%stride = map_src(udimord)%stride
    call map_set(var, map=map_result, stat=stat)
  else if (sclass == vtb_class_memory) then
    var = source_var
    stat = DC_NOERR
  else
    stat = GT_EFAKE
  endif

  deallocate(map_src)
  endsub_msg = CPrintf('result_var=%d', i=(/var%mapid/))
999 continue
  call StoreError(stat, subname, cause_i=cause_i)
  call endsub(subname, '%c', c1=trim(endsub_msg))
end subroutine GTVarOpenByDimOrd
